import { PostgrestFilterBuilder, PostgrestQueryBuilder, PostgrestError as PostgrestError$1, PostgrestResponse, PostgrestTransformBuilder } from '@supabase/postgrest-js';
import { GenericSchema, GenericTable } from '@supabase/postgrest-js/dist/module/types';
import { MutatorOptions, Key, Middleware, SWRResponse, SWRConfiguration } from 'swr';
import { PostgrestMutatorOpts, DecodedKey } from '@supabase-cache-helpers/postgrest-shared';
import { PostgrestParser, PostgrestQueryParserOptions, PostgrestFilter } from '@supabase-cache-helpers/postgrest-filter';
import * as swr_infinite from 'swr/infinite';
import { SWRInfiniteHook, SWRInfiniteKeyLoader, SWRInfiniteFetcher, SWRInfiniteConfiguration, SWRInfiniteResponse } from 'swr/infinite';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { PostgrestError, SupabaseClient, RealtimePostgresChangesFilter, REALTIME_POSTGRES_CHANGES_LISTEN_EVENT, RealtimePostgresChangesPayload, RealtimeChannel } from '@supabase/supabase-js';
import * as use_mutation from 'use-mutation';
import { Options, MutationResult } from 'use-mutation';

declare const KEY_PREFIX = "postgrest";
declare const POSTGREST_FILTER_KEY_PREFIX = "postgrest-filter";
declare const KEY_SEPARATOR = "$";
declare const INFINITE_PREFIX = "$inf$";

declare const createKeyGetter: <Schema extends GenericSchema, Table extends Record<string, unknown>, Result>(query: PostgrestFilterBuilder<Schema, Table, Result> | null, pageSize: number) => (pageIndex: number, previousPageData: Result[]) => PostgrestFilterBuilder<Schema, Table, Result> | null;

declare type PostgrestSWRMutatorOpts<T extends GenericTable> = PostgrestMutatorOpts<T["Row"]> & MutatorOptions;
declare type DecodedSWRKey = DecodedKey & {
    isInfinite: boolean;
    key: string;
};

declare const decode: (key: Key) => DecodedSWRKey | null;

declare const encode: <Result>(parser: PostgrestParser<Result>) => string;

declare const getTable: (query: PostgrestQueryBuilder<GenericSchema, GenericTable>) => string;

declare const middleware: Middleware;
declare const infiniteMiddleware: <Result>(useSWRInfiniteNext: SWRInfiniteHook) => (keyFn: SWRInfiniteKeyLoader, fetcher: SWRInfiniteFetcher, config: SWRInfiniteConfiguration) => swr_infinite.SWRInfiniteResponse<any, any>;

declare const usePostgrestFilterCache: <R extends Record<string, unknown>>() => (query: string, opts?: PostgrestQueryParserOptions) => PostgrestFilter<any>;

declare type Operation = "InsertOne" | "InsertMany" | "UpdateOne" | "UpsertOne" | "UpsertMany" | "DeleteOne";
declare type GetInputType<T extends GenericTable, O extends Operation> = O extends "DeleteOne" ? Partial<T["Row"]> : O extends "InsertOne" | "UpsertOne" ? T["Insert"] : O extends "InsertMany" | "UpsertMany" ? T["Insert"][] : O extends "UpdateOne" ? T["Update"] : never;
declare type UsePostgrestSWRMutationOpts<S extends GenericSchema, T extends GenericTable, O extends Operation, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>> = PostgrestSWRMutatorOpts<T> & Options<GetInputType<T, O>, R, PostgrestError>;

declare function useDeleteMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "DeleteOne", Q, R>): use_mutation.MutationResult<Partial<T["Row"]>, R, PostgrestError$1>;

declare function useInsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "single", primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "InsertOne", Q, R>): MutationResult<T["Insert"], T["Row"], PostgrestError$1>;
declare function useInsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "multiple", primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "InsertMany", Q, R>): MutationResult<T["Insert"][], T["Row"][], PostgrestError$1>;

declare function useUpdateMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "UpdateOne", Q, R>): use_mutation.MutationResult<T["Update"], R, PostgrestError$1>;

declare function useUpsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "single", primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "UpsertOne", Q, R>): MutationResult<T["Insert"], T["Row"], PostgrestError$1>;
declare function useUpsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(qb: PostgrestQueryBuilder<S, T>, mode: "multiple", primaryKeys: (keyof T["Row"])[], query?: Q, opts?: UsePostgrestSWRMutationOpts<S, T, "UpsertMany", Q, R>): MutationResult<T["Insert"][], T["Row"][], PostgrestError$1>;

declare type UseCountedPaginationArgs<Result extends Record<string, unknown>> = {
    pageSize: number;
} & {
    dataQuery: SWRInfiniteResponse<Result[], PostgrestError$1>;
    countQuery: SWRResponse<Result[], PostgrestError$1> & Pick<PostgrestResponse<Result[]>, "count">;
};
declare type UseCountedPaginationResponse<Data> = Pick<SWRInfiniteResponse<Data, PostgrestError$1>, "isValidating" | "error"> & {
    pages: SWRInfiniteResponse<Data[], PostgrestError$1>["data"];
    pageCount: null | number;
    pageIndex: number;
    setPage: (idx: number) => void;
    currentPage: null | Data[];
    nextPage: null | (() => void);
    previousPage: null | (() => void);
};
declare const useCountedPagination: <Result extends Record<string, unknown>>({ pageSize, countQuery, dataQuery, }: UseCountedPaginationArgs<Result>) => UseCountedPaginationResponse<Result>;

declare function useInfiniteQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestFilterBuilder<Schema, Table, Result> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): SWRInfiniteResponse<Result[], PostgrestError$1>;

declare type ArrayElement<ArrayType extends readonly unknown[]> = ArrayType extends readonly (infer ElementType)[] ? ElementType : never;
declare type SWRInfiniteScrollPostgrestResponse<Type> = Pick<SWRInfiniteResponse<Type, PostgrestError$1>, "isValidating" | "error"> & {
    loadMore: null | (() => void);
    data: ArrayElement<Exclude<SWRInfiniteResponse<Type, PostgrestError$1>["data"], undefined>> | undefined;
};
declare function useInfiniteScrollQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestFilterBuilder<Schema, Table, Result> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): SWRInfiniteScrollPostgrestResponse<Result[]>;

declare type SWRInfinitePaginationPostgrestResponse<Type> = Pick<SWRInfiniteResponse<Type, PostgrestError$1>, "isValidating" | "error"> & {
    pages: SWRInfiniteResponse<Type[], PostgrestError$1>["data"];
    currentPage: null | Type[];
    pageIndex: number;
    setPage: (idx: number) => void;
    nextPage: null | (() => void);
    previousPage: null | (() => void);
};
declare function usePaginationQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestFilterBuilder<Schema, Table, Result> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): SWRInfinitePaginationPostgrestResponse<Result>;

/**
 * Perform a postgrest query
 * @param query
 * @param mode
 * @param config
 */
declare function useQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result>(query: PostgrestFilterBuilder<Schema, Table, Result> | PostgrestTransformBuilder<Schema, Table, Result> | null, mode: "single", config?: SWRConfiguration): SWRResponse<Result, PostgrestError$1>;
declare function useQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result>(query: PostgrestFilterBuilder<Schema, Table, Result> | PostgrestTransformBuilder<Schema, Table, Result> | null, mode: "maybeSingle", config?: SWRConfiguration): SWRResponse<Result | undefined, PostgrestError$1>;
declare function useQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result>(query: PostgrestFilterBuilder<Schema, Table, Result> | PostgrestTransformBuilder<Schema, Table, Result> | null, mode: "multiple", config?: SWRConfiguration): SWRResponse<Result[], PostgrestError$1> & Pick<PostgrestResponse<Result[]>, "count">;

declare type RealtimeEventType = "INSERT" | "UPDATE" | "DELETE";
declare const isV1Response: <T extends GenericTable>(payload: unknown) => payload is ResponseV1<T>;
declare type ResponseV1<T extends GenericTable> = {
    commit_timestamp: string;
    schema: string;
    table: string;
    type: RealtimeEventType;
    columns: Column[];
    record: T["Row"];
    old_record: T["Row"];
    errors: null | string[];
};
declare type Column = {
    flags: string[];
    name: string;
    type: string;
    type_modifier: number;
};

declare function useSubscriptionQuery<S extends GenericSchema, T extends GenericTable, Q extends string = "*", R = GetResult<S, T["Row"], Q extends "*" ? "*" : Q>>(client: SupabaseClient | null, channelName: string, filter: RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, query: Q, primaryKeys: (keyof T["Row"])[], opts?: PostgrestSWRMutatorOpts<T> & {
    callback?: (event: RealtimePostgresChangesPayload<T["Row"]> & {
        data: T["Row"] | R;
    }) => void | Promise<void>;
}): {
    channel: RealtimeChannel | undefined;
    status: string | undefined;
};

declare function useSubscription<T extends GenericTable>(channel: RealtimeChannel | null, filter: RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, primaryKeys: (keyof T["Row"])[], opts?: PostgrestSWRMutatorOpts<T> & {
    callback?: (event: RealtimePostgresChangesPayload<T["Row"]>) => void | Promise<void>;
}): {
    status: string | undefined;
};

export { Column, DecodedSWRKey, GetInputType, INFINITE_PREFIX, KEY_PREFIX, KEY_SEPARATOR, Operation, POSTGREST_FILTER_KEY_PREFIX, PostgrestSWRMutatorOpts, RealtimeEventType, ResponseV1, SWRInfinitePaginationPostgrestResponse, SWRInfiniteScrollPostgrestResponse, UseCountedPaginationArgs, UseCountedPaginationResponse, UsePostgrestSWRMutationOpts, createKeyGetter, decode, encode, getTable, infiniteMiddleware, isV1Response, middleware, useCountedPagination, useDeleteMutation, useInfiniteQuery, useInfiniteScrollQuery, useInsertMutation, usePaginationQuery, usePostgrestFilterCache, useQuery, useSubscription, useSubscriptionQuery, useUpdateMutation, useUpsertMutation };
